"""
The command line module provides the main function of the application.
"""
# from asyncio import subprocess
import subprocess
import sys
import json
import re
from pathlib import Path
from os import path, getcwd
import click
from smelly_python.code_smell import Report
from .generator.webpage_generator import generate_webpage
from .generator.md_generator import generate_md


@click.command()
@click.option('--directory', '-d', type=click.Path(exists=True),
              help='Specify the python main directory for pylint.')
def main(directory):
    """
    Main command line interface.
    Takes the first command line argument to be the json generated by pylint.
    """
    if not directory:
        print("Please provide the --dir parameter")
        sys.exit(1)
    _setup_dirs()
    print('Running pylint...')
    result = None
    try:
        result = subprocess.run(['pylint', directory, f'--output-format=json:'
        f'{_get_reports("report.json")},text:{_get_reports("grade.txt")}', '--exit-zero'],
                                capture_output=True, text=True, check=True)
    except subprocess.CalledProcessError:
        print(
            f'Whoops we could not run pylint for the following directory: {directory}')
        print(result.stderr)
        sys.exit(1)
    print('Finished running python, creating report...')
    with open(_get_reports('report.json'), 'r', encoding='utf-8') as input_file:
        content = json.load(input_file)
    with open(_get_reports('grade.txt'), 'r', encoding='utf-8') as input_file:
        grade = re.search(
            r"Your code has been rated at (\d+\.?\d*)", input_file.read()).group(1)

    report = Report(content, grade)
    generate_webpage(report)
    generate_md(report)


def _setup_dirs():
    Path(path.join('report', 'smelly_python')).mkdir(parents=True, exist_ok=True)


def _get_reports(file_name):
    return path.join(getcwd(), 'report/smelly_python', file_name)
